@@ -3,7 +3,7 @@
 # Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
 # Copyright (C) 2015 Brad Cowie, Christopher Lorier and Joe Stringer.
 # Copyright (C) 2015 Research and Education Advanced Network New Zealand Ltd.
-# Copyright (C) 2015--2018 The Contributors
+# Copyright (C) 2015--2019 The Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@ import os
 import select
 import socket
 import sys
+import time
 
 import pika
 
@@ -82,7 +83,7 @@ class RabbitAdapter:
         """Make connection to rabbit to send events"""
         # check if a rabbit host was specified
         if not self.host:
-            print('Not connecting to any RabbitMQ, host is None.')
+            print('Not connecting to any RabbitMQ, host is None.')  # pylint: disable=print-statement
             return False
 
         # create connection to rabbit
@@ -94,22 +95,18 @@ class RabbitAdapter:
             self.channel = pika.BlockingConnection(params).channel()
             self.channel.exchange_declare(exchange=self.exchange,
                                           exchange_type=self.exchange_type)
-        except pika.exceptions.AMQPError as err:
+        except (pika.exceptions.AMQPError, socket.gaierror, OSError) as err:
             print("Unable to connect to RabbitMQ at %s:%s because: %s" %
-                  (self.host, self.port, err))
+                  (self.host, self.port, err))  # pylint: disable=print-statement
             return False
-        except socket.gaierror as err:
-            print("Unable to connect to RabbitMQ at %s:%s because: %s" %
-                  (self.host, self.port, err))
-            return False
-        print("Connected to RabbitMQ at %s:%s" % (self.host, self.port))
+        print("Connected to RabbitMQ at %s:%s" % (self.host, self.port))  # pylint: disable=print-statement
         return True
 
     def socket_conn(self):
         """Make connection to sock to receive events"""
         # check if socket events are enabled
         if self.event_sock == '0':
-            print('Not connecting to any socket, FA_EVENT_SOCK is none.')
+            print('Not connecting to any socket, FAUCET_EVENT_SOCK is none.')  # pylint: disable=print-statement
             return False
         if self.event_sock == '1':
             self.event_sock = get_sys_prefix() + '/var/run/faucet/faucet.sock'
@@ -118,50 +115,55 @@ class RabbitAdapter:
         # create connection to unix socket
         try:
             self.sock.connect(self.event_sock)
+            self.sock.setblocking(False)
         except socket.error as err:
-            print("Failed to connect to the socket because: %s" % err)
+            print("Failed to connect to the socket because: %s" % err)  # pylint: disable=print-statement
             return False
-        print("Connected to the socket at %s" % self.event_sock)
+        print("Connected to the socket at %s" % self.event_sock)  # pylint: disable=print-statement
         return True
 
+    def poll_events(self):
+        """Poll FAUCET socket for events."""
+        socket_ok = False
+        event_buffer = b''
+        read_ready, _, _ = select.select([self.sock], [], [])
+        if self.sock in read_ready:
+            socket_ok = True
+            while True:
+                try:
+                    event_buffer += self.sock.recv(1024)
+                except socket.error as err:
+                    socket_ok = err.errno == errno.EWOULDBLOCK
+                    break
+
+        events = event_buffer.strip().split(b'\n')
+        return (socket_ok, events)
+
     def main(self):
         """Make connections to sock and rabbit and receive messages from sock
         to sent to rabbit
         """
         # ensure connections to the socket and rabbit before getting messages
         if self.rabbit_conn() and self.socket_conn():
-            # get events from socket
-            self.sock.setblocking(False)
-            recv_data = True
-            buffer = b''
-            while recv_data:
-                if not buffer:
-                    read_ready, _, _ = select.select([self.sock], [], [])
-                    if self.sock in read_ready:
-                        continue_recv = True
-                        while continue_recv:
-                            try:
-                                buffer += self.sock.recv(1024)
-                            except socket.error as err:
-                                if err.errno != errno.EWOULDBLOCK:
-                                    recv_data = False
-                                continue_recv = False
-                # send events to rabbit
+            socket_ok = True
+            events = []
+            while socket_ok:
+                if not events:
+                    socket_ok, events = self.poll_events()
                 try:
-                    buffers = buffer.strip().split(b'\n')
-                    for buff in buffers:
-                        self.channel.basic_publish(exchange=self.exchange,
-                                                   routing_key=self.routing_key,
-                                                   body=buff,
-                                                   properties=pika.BasicProperties(
-                                                       delivery_mode=2,
-                                                   ))
-                    buffer = b''
+                    for event in events:
+                        self.channel.basic_publish(
+                            exchange=self.exchange,
+                            routing_key=self.routing_key,
+                            body=event,
+                            properties=pika.BasicProperties(delivery_mode=2,))
+                    events = []
                 except pika.exceptions.AMQPError as err:
-                    print("Unable to send event to RabbitMQ because: %s" % err)
-                    print("The following event will be retried: %r" % buffer)
+                    print("Unable to send events %s to RabbitMQ: %s, retrying" % (  # pylint: disable=print-statement
+                        events, err))
+                    time.sleep(1)
                     self.rabbit_conn()
-                sys.stdout.flush()
+                    sys.stdout.flush()
             self.sock.close()
 
 
