@@ -529,10 +529,6 @@ configuration.
         if 'egress_acl' in included_tables:
             table_configs['eth_dst'].miss_goto = 'egress_acl'
 
-        if 'copro' in included_tables:
-            if 'vlan_acl' in included_tables:
-                table_configs['copro'].miss_goto = 'vlan_acl'
-
         oxm_fields = set(valve_of.MATCH_FIELDS.keys())
 
         for table_name, table_config in table_configs.items():
@@ -1096,7 +1092,7 @@ configuration.
                         mirror_from_port[mirrored_port].append(mirror_port)
 
             # TODO: confusingly, mirror at config time means what ports to mirror from.
-            # But internally we use as a list of ports to mirror to.
+            #   But internally we use as a list of ports to mirror to.
             for mirrored_port, mirror_ports in mirror_from_port.items():
                 mirrored_port.mirror = []
                 for mirror_port in mirror_ports:
