@@ -51,6 +51,9 @@ class FaucetMultiDPTest(FaucetTopoTestBase):
         """Host index or (switch index, switch index) link to acls_in mapping"""
         return {}
 
+    def output_only(self):
+        return set()
+
     def setUp(self):
         pass
 
@@ -104,17 +107,19 @@ class FaucetMultiDPTest(FaucetTopoTestBase):
         # Create host link topology and vlan information
         host_links = {}
         host_vlans = {}
+        tagged_vlans = list(range(n_vlans))
         host = 0
         for dp_i in range(n_dps):
             for _ in range(n_tagged):
                 host_links[host] = [dp_i]
-                vlans = list(range(n_vlans))
-                host_vlans[host] = vlans
+                host_vlans[host] = tagged_vlans
                 host += 1
             for _ in range(n_untagged):
                 host_links[host] = [dp_i]
                 host_vlans[host] = 0
                 host += 1
+        for host in self.output_only():
+            host_vlans[host] = None
         # Create Host configuration options for DP interfaces
         host_options = {}
         if use_external:
@@ -381,7 +386,7 @@ class FaucetStringOfDPLACPUntaggedTest(FaucetMultiDPTest):
         interfaces_conf[fail_port]['lacp'] = 0
         interfaces_conf[fail_port]['lacp_active'] = False
         self.reload_conf(conf, self.faucet_config_path, restart=True,
-                         cold_start=False, change_expected=False)
+                         cold_start=False, change_expected=True)
 
         self.wait_for_lacp_port_init(src_port, self.dpids[0], self.topo.switches_by_id[0])
         self.wait_for_lacp_port_up(dst_port, self.dpids[0], self.topo.switches_by_id[0])
@@ -403,7 +408,7 @@ class FaucetStringOfDPLACPUntaggedTest(FaucetMultiDPTest):
         interfaces_conf[end_port]['lacp'] = 2
 
         self.reload_conf(conf, self.faucet_config_path, restart=True,
-                         cold_start=False, change_expected=False)
+                         cold_start=True, change_expected=True)
 
         self.wait_for_all_lacp_up()
         self.verify_stack_hosts()
@@ -411,7 +416,7 @@ class FaucetStringOfDPLACPUntaggedTest(FaucetMultiDPTest):
         interfaces_conf[fail_port]['lacp'] = 0
         interfaces_conf[fail_port]['lacp_active'] = False
         self.reload_conf(conf, self.faucet_config_path, restart=True,
-                         cold_start=False, change_expected=False)
+                         cold_start=False, change_expected=True)
 
         self.wait_for_lacp_port_init(src_port, self.dpids[0], self.topo.switches_by_id[0])
         self.wait_for_lacp_port_up(dst_port, self.dpids[0], self.topo.switches_by_id[0])
@@ -1051,7 +1056,7 @@ class FaucetTunnelSameDpTest(FaucetMultiDPTest):
 
 
 class FaucetSingleTunnelTest(FaucetMultiDPTest):
-    """Test the Faucet tunnel ACL option"""
+    """Test the Faucet tunnel ACL option both locally and remotely with link failure"""
 
     NUM_DPS = 2
     NUM_HOSTS = 2
@@ -1076,16 +1081,34 @@ class FaucetSingleTunnelTest(FaucetMultiDPTest):
                                 'port': self.host_port_maps[2][1][0]}
                         }
                     }
-                }}
+                }},
+                {'rule': {
+                    'dl_type': IPV6_ETH,
+                    'ip_proto': 56,
+                    'actions': {
+                        'allow': 0,
+                        'output': {
+                            'tunnel': {
+                                'type': 'vlan',
+                                'tunnel_id': 200,
+                                'dp': self.topo.switches_by_id[1],
+                                'port': self.host_port_maps[2][1][0]}
+                        }
+                    }
+                }},
             ]
         }
 
     def link_acls(self):
         """DP-to-acl port mapping"""
         return {
-            0: [1]  # Host 0 'acls_in': [1]
+            0: [1],  # Host 0 'acls_in': [1]
+            3: [1],  # Host 3 'acls_in': [1]
         }
 
+    def output_only(self):
+        return {2}   # Host 2 (first port, second switch).
+
     def setUp(self):  # pylint: disable=invalid-name
         """Start the network"""
         super(FaucetSingleTunnelTest, self).set_up(
@@ -1094,25 +1117,21 @@ class FaucetSingleTunnelTest(FaucetMultiDPTest):
             n_untagged=self.NUM_HOSTS,
             switch_to_switch_links=self.SWITCH_TO_SWITCH_LINKS)
 
-    def test_tunnel_established(self):
-        """Test a tunnel path can be created."""
-        self.verify_stack_up()
-        src_host = self.net.get(self.topo.hosts_by_id[0])
-        dst_host = self.net.get(self.topo.hosts_by_id[2])
+    def verify_tunnels(self):
+        """Test tunnel connectivity from local and remote switches."""
         other_host = self.net.get(self.topo.hosts_by_id[1])
-        self.verify_tunnel_established(src_host, dst_host, other_host)
+        dst_host = self.net.get(self.topo.hosts_by_id[2])
+        for src_host_id in (0, 3):
+            src_host = self.net.get(self.topo.hosts_by_id[src_host_id])
+            self.verify_tunnel_established(src_host, dst_host, other_host, packets=10)
 
     def test_tunnel_path_rerouted(self):
-        """Test a tunnel path is rerouted when a link is down."""
+        """Test remote tunnel path is rerouted when a link is down."""
         self.verify_stack_up()
-        src_host = self.net.get(self.topo.hosts_by_id[0])
-        dst_host = self.net.get(self.topo.hosts_by_id[2])
-        other_host = self.net.get(self.topo.hosts_by_id[1])
-        self.verify_tunnel_established(src_host, dst_host, other_host, packets=10)
+        self.verify_tunnels()
         first_stack_port = self.link_port_maps[(0, 1)][0]
         self.one_stack_port_down(self.dpids[0], self.topo.switches_by_id[0], first_stack_port)
-        src_host, other_host, dst_host = self.hosts_name_ordered()[:3]
-        self.verify_tunnel_established(src_host, dst_host, other_host, packets=10)
+        self.verify_tunnels()
 
 
 class FaucetTunnelLoopTest(FaucetSingleTunnelTest):
@@ -2206,8 +2225,9 @@ class FaucetStackWarmStartTest(FaucetTopoTestBase):
         del conf['dps'][self.topo.switches_by_id[2]]['interfaces'][self.link_port_maps[(2, 1)][0]]
         self.reload_conf(
             conf, self.faucet_config_path, restart=True,
-            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[1])
-        self.verify_stack_up(timeout=1)
+            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[0])
+        # Due to flood table size changes, some DPs will be cold starting
+        self.verify_stack_up(timeout=1, prop=0.5)
         self.verify_intervlan_routing()
 
     def test_del_primary_stack_port(self):
@@ -2220,8 +2240,9 @@ class FaucetStackWarmStartTest(FaucetTopoTestBase):
         del conf['dps'][self.topo.switches_by_id[2]]['interfaces'][self.link_port_maps[(2, 1)][1]]
         self.reload_conf(
             conf, self.faucet_config_path, restart=True,
-            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[1])
-        self.verify_stack_up(timeout=1)
+            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[0])
+        # Due to flood table size changes, some DPs will be cold starting
+        self.verify_stack_up(timeout=1, prop=0.5)
         self.verify_intervlan_routing()
 
     def test_del_host(self):
@@ -2235,8 +2256,8 @@ class FaucetStackWarmStartTest(FaucetTopoTestBase):
         del interfaces_conf[self.host_port_maps[0][0][0]]
         self.reload_conf(
             conf, self.faucet_config_path, restart=True,
-            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[0])
-        self.verify_stack_up(timeout=1)
+            cold_start=True, change_expected=True, dpid=self.topo.dpids_by_id[0])
+        self.verify_stack_up()
         del self.host_information[0]
         self.verify_intervlan_routing()
 
@@ -2260,7 +2281,7 @@ class FaucetStackWarmStartTest(FaucetTopoTestBase):
             'stack': {'dp': self.topo.switches_by_id[0], 'port': port_num}}
         self.reload_conf(
             conf, self.faucet_config_path, restart=True,
-            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[1])
+            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[2])
         self.verify_stack_up()
         self.verify_intervlan_routing()
 
@@ -2284,7 +2305,7 @@ class FaucetStackWarmStartTest(FaucetTopoTestBase):
             'stack': {'dp': self.topo.switches_by_id[1], 'port': port_num}}
         self.reload_conf(
             conf, self.faucet_config_path, restart=True,
-            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[1])
+            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[0])
         self.verify_stack_up()
         self.verify_intervlan_routing()
 
@@ -2309,6 +2330,6 @@ class FaucetStackWarmStartTest(FaucetTopoTestBase):
         # Expected cold start as topology changed with all ports being stack ports
         self.reload_conf(
             conf, self.faucet_config_path, restart=True,
-            cold_start=True, change_expected=True, dpid=self.topo.dpids_by_id[1])
+            cold_start=False, change_expected=True, dpid=self.topo.dpids_by_id[0])
         self.verify_stack_up()
         self.verify_intervlan_routing()
