@@ -31,8 +31,6 @@ from faucet.port import (
     STACK_STATE_INIT, STACK_STATE_UP,
     LACP_PORT_SELECTED, LACP_PORT_UNSELECTED)
 
-from mininet.topo import Topo  # pylint: disable=unused-import
-
 from clib.fakeoftable import CONTROLLER_PORT
 
 from clib.valve_test_lib import (
@@ -110,13 +108,13 @@ dps:
         valve = self.valves_manager.valves[0x1]
         other_valves = self.get_other_valves(valve)
         # Equal number of LAG ports, choose root DP
-        nominated_dpid = valve.get_lacp_dpid_nomination(1, other_valves)[0]
+        nominated_dpid = valve.switch_manager.get_lacp_dpid_nomination(1, valve, other_valves)[0]
         self.assertEqual(
             nominated_dpid, 0x1,
             'Expected nominated DPID %s but found %s' % (0x1, nominated_dpid))
         # Choose DP with most UP LAG ports
         lacp_ports[0x1][0].actor_nosync()
-        nominated_dpid = valve.get_lacp_dpid_nomination(1, other_valves)[0]
+        nominated_dpid = valve.switch_manager.get_lacp_dpid_nomination(1, valve, other_valves)[0]
         self.assertEqual(
             nominated_dpid, 0x2,
             'Expected nominated DPID %s but found %s' % (0x2, nominated_dpid))
@@ -127,7 +125,7 @@ dps:
         valve = self.valves_manager.valves[0x1]
         other_valves = self.get_other_valves(valve)
         # No actors UP so should return None
-        nominated_dpid = valve.get_lacp_dpid_nomination(1, other_valves)[0]
+        nominated_dpid = valve.switch_manager.get_lacp_dpid_nomination(1, valve, other_valves)[0]
         self.assertEqual(
             nominated_dpid, None,
             'Did not expect to nominate DPID %s' % nominated_dpid)
@@ -136,7 +134,7 @@ dps:
             for port in valve.dp.ports.values():
                 if port.lacp:
                     port.actor_up()
-        nominated_dpid = valve.get_lacp_dpid_nomination(1, None)[0]
+        nominated_dpid = valve.switch_manager.get_lacp_dpid_nomination(1, valve, None)[0]
         self.assertEqual(
             nominated_dpid, None,
             'Did not expect to nominate DPID %s' % nominated_dpid)
@@ -154,7 +152,8 @@ dps:
         for valve, ports in lacp_ports.items():
             other_valves = self.get_other_valves(valve)
             for port in ports:
-                valve.lacp_update_port_selection_state(port, other_valves)
+                valve.switch_manager.lacp_update_port_selection_state(port, valve, other_valves)
+                valve._reset_lacp_status(port)
                 # Testing accuracy of varz port_lacp_role
                 port_labels = {
                     'port': port.name,
@@ -856,11 +855,15 @@ class ValveStackRedundancyTestCase(ValveTestBases.ValveTestNetwork):
         self.assertFalse(self.valves_manager.maintain_stack_root(now))
         self.assertEqual('s1', self.valves_manager.meta_dp_state.stack_root_name)
         self.assertEqual(1, self.get_prom('faucet_stack_root_dpid', bare=True))
+        self.assertTrue(self.get_prom('is_dp_stack_root', dp_id=1))
+        self.assertFalse(self.get_prom('is_dp_stack_root', dp_id=2))
         now += (valves_manager.STACK_ROOT_DOWN_TIME * 2)
         # Time passes, still no change, s1 is still the root.
         self.assertFalse(self.valves_manager.maintain_stack_root(now))
         self.assertEqual('s1', self.valves_manager.meta_dp_state.stack_root_name)
         self.assertEqual(1, self.get_prom('faucet_stack_root_dpid', bare=True))
+        self.assertTrue(self.get_prom('is_dp_stack_root', dp_id=1))
+        self.assertFalse(self.get_prom('is_dp_stack_root', dp_id=2))
         # s2 has come up, but has all stack ports down and but s1 is still down.
         self.valves_manager.meta_dp_state.dp_last_live_time['s2'] = now
         now += (valves_manager.STACK_ROOT_STATE_UPDATE_TIME * 2)
@@ -873,6 +876,8 @@ class ValveStackRedundancyTestCase(ValveTestBases.ValveTestNetwork):
         self.assertTrue(self.valves_manager.maintain_stack_root(now))
         self.assertEqual('s2', self.valves_manager.meta_dp_state.stack_root_name)
         self.assertEqual(2, self.get_prom('faucet_stack_root_dpid', bare=True))
+        self.assertFalse(self.get_prom('is_dp_stack_root', dp_id=1))
+        self.assertTrue(self.get_prom('is_dp_stack_root', dp_id=2))
         # More time passes, s1 is still down, s2 is still the root.
         now += (valves_manager.STACK_ROOT_DOWN_TIME * 2)
         # s2 recently said something, s2 still the root.
@@ -881,12 +886,16 @@ class ValveStackRedundancyTestCase(ValveTestBases.ValveTestNetwork):
         self.assertFalse(self.valves_manager.maintain_stack_root(now))
         self.assertEqual('s2', self.valves_manager.meta_dp_state.stack_root_name)
         self.assertEqual(2, self.get_prom('faucet_stack_root_dpid', bare=True))
+        self.assertFalse(self.get_prom('is_dp_stack_root', dp_id=1))
+        self.assertTrue(self.get_prom('is_dp_stack_root', dp_id=2))
         # now s1 came up too, but we stay on s2 because it's healthy.
         self.valves_manager.meta_dp_state.dp_last_live_time['s1'] = now + 1
         now += valves_manager.STACK_ROOT_STATE_UPDATE_TIME
         self.assertFalse(self.valves_manager.maintain_stack_root(now))
         self.assertEqual('s2', self.valves_manager.meta_dp_state.stack_root_name)
         self.assertEqual(2, self.get_prom('faucet_stack_root_dpid', bare=True))
+        self.assertFalse(self.get_prom('is_dp_stack_root', dp_id=1))
+        self.assertTrue(self.get_prom('is_dp_stack_root', dp_id=2))
 
 
 class ValveRootStackTestCase(ValveTestBases.ValveTestNetwork):
@@ -2613,79 +2622,5 @@ dps:
                     'Stack port detected as changed on non-topology change')
 
 
-class ValveNetworkTest(ValveTestBases.ValveTestNetwork):
-    """Test an auto-generated path topology and FakeOFNetwork packet traversal"""
-
-    topo = None
-
-    NUM_DPS = 2
-    NUM_VLANS = 1
-    NUM_HOSTS = 1
-    SWITCH_TO_SWITCH_LINKS = 1
-
-    def setUp(self):
-        """Setup auto-generated network topology and trigger stack ports"""
-        self.topo, self.CONFIG = self.create_topo_config(networkx.path_graph(self.NUM_DPS))
-        self.setup_valves(self.CONFIG)
-        self.trigger_stack_ports()
-
-    def test_network(self):
-        """Test packet output to the adjacent switch"""
-        _, host_port_maps, _ = self.topo.create_port_maps()
-        vlan_vid = self.topo.vlan_vid(0) | ofp.OFPVID_PRESENT
-        bcast_match = {
-            'in_port': host_port_maps[0][0][0],
-            'eth_src': '00:00:00:00:00:12',
-            'eth_dst': mac.BROADCAST_STR,
-            'ipv4_src': '10.1.0.1',
-            'ipv4_dst': '10.1.0.2',
-            'vlan_vid': vlan_vid
-        }
-        self.assertTrue(
-            self.network.is_output(bcast_match, 1, 2, host_port_maps[1][1][0], vlan_vid))
-
-
-class ValveLoopNetworkTest(ValveTestBases.ValveTestNetwork):
-    """Test an auto-generated loop topology and FakeOFNetwork packet traversal"""
-
-    topo = None
-
-    NUM_DPS = 3
-    NUM_VLANS = 1
-    NUM_HOSTS = 1
-    SWITCH_TO_SWITCH_LINKS = 2
-
-    def setUp(self):
-        """Setup auto-generated network topology and trigger stack ports"""
-        self.topo, self.CONFIG = self.create_topo_config(networkx.cycle_graph(self.NUM_DPS))
-        self.setup_valves(self.CONFIG)
-        self.trigger_stack_ports()
-
-    def test_network(self):
-        """Test packet output to the adjacent switch in a loop topology"""
-        _, host_port_maps, link_port_maps = self.topo.create_port_maps()
-        vlan_vid = self.topo.vlan_vid(0) | ofp.OFPVID_PRESENT
-        bcast_match = {
-            'in_port': host_port_maps[0][0][0],
-            'eth_src': '00:00:00:00:00:12',
-            'eth_dst': mac.BROADCAST_STR,
-            'ipv4_src': '10.1.0.1',
-            'ipv4_dst': '10.1.0.2',
-            'vlan_vid': vlan_vid
-        }
-        self.assertTrue(
-            self.network.is_output(bcast_match, 1, 3, host_port_maps[1][1][0], vlan_vid))
-        self.assertTrue(
-            self.network.is_output(bcast_match, 1, 2, host_port_maps[1][1][0], vlan_vid))
-        port_num = link_port_maps[(0, 1)][0]
-        port = self.valves_manager.valves[1].dp.ports[port_num]
-        reverse_port = port.stack['port']
-        self.trigger_stack_ports([port, reverse_port])
-        self.assertTrue(
-            self.network.is_output(bcast_match, 1, 3, host_port_maps[1][1][0], vlan_vid))
-        self.assertTrue(
-            self.network.is_output(bcast_match, 1, 2, host_port_maps[1][1][0], vlan_vid))
-
-
 if __name__ == "__main__":
     unittest.main()  # pytype: disable=module-attr
